@@ -1,12 +1,11 @@
 ;;; notmuch-indicator.el --- Display mode line indicator with notmuch-count(1) output -*- lexical-binding: t -*-
 
-;; Copyright (C) 2022-2023  Free Software Foundation, Inc.
+;; Copyright (C) 2022-2024  Free Software Foundation, Inc.
 
 ;; Author: Protesilaos Stavrou <info@protesilaos.com>
-;; Maintainer: notmuch-indicator Development <~protesilaos/general-issues@lists.sr.ht>
-;; URL: https://git.sr.ht/~protesilaos/notmuch-indicator
-;; Mailing-List: https://lists.sr.ht/~protesilaos/general-issues
-;; Version: 1.1.0
+;; Maintainer: Protesilaos Stavrou <info@protesilaos.com>
+;; URL: https://github.com/protesilaos/notmuch-indicator
+;; Version: 1.2.0
 ;; Package-Requires: ((emacs "27.1"))
 ;; Keywords: convenience, mail
 
@@ -28,49 +27,20 @@
 ;;; Commentary:
 ;;
 ;; This is a simple package that renders an indicator with an email
-;; count of the `notmuch' index on the Emacs mode line.  The underlying
-;; mechanism is that of `notmuch-count(1)', which is used to find the
-;; number of items that match the given search terms.
+;; count of the `notmuch' index on the Emacs mode line.  The
+;; underlying mechanism is that of `notmuch-count(1)', which is used
+;; to find the number of items that match the given search terms.  For
+;; example, the letter "U" or the emoji "💬" can accompany search
+;; terms for "unread items".  The resulting indicator may optionally
+;; be styled with a face, only for the label, only for the counter, or
+;; both.
 ;;
-;; The indicator is enabled when `notmuch-indicator-mode' is on.
+;; The indicator is displayed when `notmuch-indicator-mode' is enabled.
 ;;
 ;; The user option `notmuch-indicator-args' provides the means to define
 ;; search terms and associate them with a given label.  The label is
 ;; purely cosmetic, though it helps characterise the resulting counter.
 ;;
-;; The value of `notmuch-indicator-args' is a list of plists (property
-;; lists).  Each plist consists of one mandatory property and two
-;; optional ones:
-;;
-;; 1. The `:terms', which is required, is a string that holds the
-;;    command-line arguments passed to `notmuch-count(1)' (read the
-;;    Notmuch documentation for the technicalities).
-;;
-;; 2. The `:label', which is optional, is an arbitrary string that is
-;;    prepended to the return value of the above.  If nil or omitted, no
-;;    label is displayed.
-;;
-;; 3. The `face', which is optional, is the symbol of a face that is
-;;    applied to the `:label'.  It should not be quoted, so like :face
-;;    bold.  Good candidates are `bold', `italic', `success', `warning',
-;;    `error', though anything will do.  If nil or omitted, no face is
-;;    used.
-;;
-;; Multiple plist lists represent separate `notmuch-count(1)' queries.
-;; These are run sequentially.  Their return values are joined into a
-;; single string.
-;;
-;; For instance, a value like the following defines three searches:
-;;
-;;     (setq notmuch-indicator-args
-;;           '((:terms "tag:unread and tag:inbox" :label "@")
-;;             (:terms "from:bank and tag:bills" :label "😱")
-;;             (:terms "--output threads tag:loveletter" :label "💕")))
-;;
-;; These form a string which realistically is like: @50 😱1000 💕0.
-;; Each component is clickable: it runs `notmuch-search' on the
-;; applicable `:terms'.
-;;
 ;; The user option `notmuch-indicator-refresh-count' determines how
 ;; often the indicator will be refreshed.  It accepts a numeric argument
 ;; which represents seconds.
@@ -82,6 +52,10 @@
 ;; The user option `notmuch-indicator-hide-empty-counters' hides zero
 ;; counters from the indicator, when it is set to a non-nil value.
 ;;
+;; The user option `notmuch-indicator-add-to-mode-line-misc-info' can
+;; be set to nil for those who want to control the placement of the
+;; `notmuch-indicator-mode-line-construct'.
+;;
 ;; Finally, and albeit obvious, the backronym for this package is:
 ;; notmuch-... Interested in Notmuch Data Indicators that Count Any
 ;; Terms Ordinarily Requested.
@@ -97,7 +71,7 @@
 (defcustom notmuch-indicator-args '((:terms "tag:unread and tag:inbox" :label "@"))
   "List of plists specifying terms for `notmuch-count(1)'.
 
-Each plist consists of one mandarory property and two optional
+Each plist consists of one mandarory property and three optional
 ones:
 
 1. The `:terms', which is required, is a string that holds the
@@ -105,34 +79,66 @@ ones:
    Notmuch documentation for the technicalities).
 
 2. The `:label', which is optional, is an arbitrary string that
-   is prepended to the return value of the above.  If nil or
-   omitted, no label is displayed.
+   is prepended to the return value of the above.  If the value
+   is nil or the property is omitted, no label is displayed.
 
-3. The `face', which is optional, is the symbol of a face that is
-   applied to the `:label'.  It should not be quoted, so like
-   :face bold.  Good candidates are `bold', `italic', `success',
-   `warning', `error', though anything will do.  If nil or
-   omitted, no face is used.
+3. The `:label-face', which is optional, is the symbol of a face
+   that is applied to the `:label'.  It should not be quoted, so
+   like :face bold.  Good candidates are `bold', `italic',
+   `success', `warning', `error', though anything will do.  If
+   the value is nil or the property is omitted, no face is used.
+   For backward-compatibility, `:face' has the same meaning as
+   `:label-face'.
+
+4. The `:counter-face', which is optional, is like `:label-face'
+   but applies to the number of the given counter.  It accepts
+   the unquoted symbol of a face, as noted above, though it also
+   takes a `inherit' value which means to use the same face as
+   the `:label-face'.  This too is an unquoted symbol.  If the
+   value is nil, or the property is omitted altogether, the
+   counter does not have a face assigned to it.
 
 Multiple plist lists represent separate `notmuch-count(1)'
 queries.  These are run sequentially.  Their return values are
-joined into a single string.
+joined into a list of strings.  By default, this is shown on the
+mode line, wherever `mode-line-misc-info' is displayed.  Refer to
+the user option `notmuch-indicator-add-to-mode-line-misc-info'
+to control the placement of the `notmuch-indicator-mode-line-construct'.
 
-For instance, a value like the following defines two commands (in
-the source code the quotes are escaped---please check the Help
-buffer for the clean code (I dislike markup in doc strings)):
+For instance, a value like the following defines three
+searches (in the source code the quotes are escaped---please
+check the Help buffer for the clean code (I dislike markup in doc
+strings)):
 
     (setq notmuch-indicator-args
          \\='((:terms \"tag:unread and tag:inbox\" :label \"@\")
-            (:terms \"from:bank and tag:bills\" :label \"😱\")
-            (:terms \"--output threads tag:loveletter\" :label \"💕\")))
+            (:terms \"from:bank and tag:bills and tag:unpaid\" :label \"😱\")
+            (:terms \"--output threads tag:fans\" :label \"❤️\")))
 
-These form a string which realistically is like: @50 😱1000 💕0."
+These form a list of strings which appears as something like
+this: @2 😱1000 ❤️0 (sorry, the `notmuch-indicator' is no miracle
+worker: your fan mail will not be more than your unpaid bills).
+
+Same idea as above, but with faces applied:
+
+    (setq notmuch-indicator-args
+         \\='(( :terms \"tag:unread and tag:inbox\"
+             :label \"@\"
+             :label-face success)
+           ( :terms \"from:bank and tag:bills and tag:unpaid\"
+             :label \"😱\"
+             :counter-face warning)
+           ( :terms \"--output threads tag:fans\"
+             :label \"❤️\"
+             :label-face error
+             :counter-face inherit)))"
   :type '(repeat
           (plist :options
                  (((const :tag "Search terms for `notmuch-count(1)'" :terms) string)
                   ((const :tag "Cosmetic label for the counter" :label) string)
-                  ((const :tag "Face applied to the label" :face) face))))
+                  ((const :tag "Face applied to the label" :label-face) face)
+                  ((const :tag "Face applied to the counter" :counter-face) face))))
+  :package-version '(notmuch-indicator . "1.2.0")
   :group 'notmuch-indicator)
 
 (defcustom notmuch-indicator-hide-empty-counters nil
@@ -201,43 +207,82 @@ package more flexible)."
   :package-version '(notmuch-indicator . "1.1.0")
   :group 'notmuch-indicator)
 
+(defcustom notmuch-indicator-add-to-mode-line-misc-info t
+  "When non-nil, append the notmuch indicator to the mode line.
+Experienced users can set this to a nil value and then include
+the `notmuch-indicator-mode-line-construct' anywhere they want in
+`mode-line-format' or related.  Also read the manual for an
+example that uses the `tab-bar-mode'."
+  :type 'boolean
+  :package-version '(notmuch-indicator . "1.2.0")
+  :group 'notmuch-indicator)
+
 ;;;; Helper functions and the minor-mode
 
 (defun notmuch-indicator--shell-command (terms)
   "Run shell command for `notmuch-count(1)' with TERMS."
   (replace-regexp-in-string
-   "\n" " "
-   (shell-command-to-string
-    (format "%s --config=%S count %s"
-            notmuch-indicator-notmuch-binary
-            notmuch-indicator-notmuch-config-file
-            terms))))
+   "\n" ""
+   (let ((default-directory "~"))
+     (shell-command-to-string
+      (format "%s --config=%S count %s"
+              notmuch-indicator-notmuch-binary
+              notmuch-indicator-notmuch-config-file
+              (shell-quote-argument terms))))))
 
 (declare-function
  notmuch-search "notmuch"
  (&optional query oldest-first target-thread target-line no-display))
 
-(defun notmuch-indicator--format-label (label count face terms)
-  "Format LABEL, COUNT, FACE and TERMS of `notmuch-indicator-args'."
+(defvar notmuch-indicator-counter-format "%s%s"
+  "The `format' string for each counter.
+It accepts two %s specifiers for the label and number,
+respectively.")
+
+(defun notmuch-indicator--format-label (label count label-face counter-face terms)
+  "Format `notmuch-indicator-args'.
+LABEL is the value of the `:label' property.  If nil, return an
+empty string.
+
+COUNT is the return value of the search TERMS of property
+`:terms'.  If nil, return an empty string.
+
+LABEL-FACE is the value of the `:label-face' property, while
+COUNTER-FACE is that of `:counter-face'.  Apply them to LABEL and
+COUNT, respectively.  If nil, do not propertize LABEL or COUNT
+with a face."
   (let ((map (make-sparse-keymap)))
     (define-key map [mode-line mouse-1]
                 (lambda () (interactive) (notmuch-search terms)))
-    (propertize
-     (format "%s%s"
-             (if (and face label)
-                 (propertize label 'face face)
-               (or label ""))
-             count)
-     'mouse-face 'mode-line-highlight
-     'help-echo (format "mouse-1: Open notmuch search for `%s'" terms)
-     'local-map map)))
+    (concat " " ; to separate multiple counters without changing the mouse hover highlight
+            (propertize
+             (format notmuch-indicator-counter-format
+                     (if (and label-face label)
+                         (propertize label 'face label-face)
+                       (or label ""))
+                     (if (and counter-face count)
+                         (propertize count 'face counter-face)
+                       (or count "")))
+             'mouse-face 'mode-line-highlight
+             'help-echo (format "mouse-1: Open notmuch search for `%s'" terms)
+             'local-map map))))
+
+(defun notmuch-indicator--get-counter-face (properties)
+  "Get :counter-face from PROPERTIES.
+If its value is `inherit', get the `:label-face'."
+  (let ((value (plist-get properties :counter-face)))
+    (if (eq value 'inherit)
+        (or (plist-get properties :label-face)
+            (plist-get properties :face))
+      value)))
 
 (defun notmuch-indicator--format-counter (count properties)
   "Format counter with COUNT and PROPERTIES of `notmuch-indicator-args'."
   (notmuch-indicator--format-label
    (plist-get properties :label)
    count
-   (plist-get properties :face)
+   (or (plist-get properties :label-face) (plist-get properties :face))
+   (notmuch-indicator--get-counter-face properties)
    (plist-get properties :terms)))
 
 (defun notmuch-indicator--get-counters ()
@@ -251,31 +296,25 @@ package more flexible)."
                (notmuch-indicator--format-counter count properties))))
          notmuch-indicator-args)))
 
-(defun notmuch-indicator--return-single-string ()
-  "Parse `notmuch-indicator-args' and format them as single string."
-  (mapconcat #'identity (notmuch-indicator--get-counters) " "))
-
-(defvar notmuch-indicator-string ""
-  "String showing the `notmuch-indicator' state.
-It is appended to the `global-mode-string'.")
-(put 'notmuch-indicator-string 'risky-local-variable t)
+(defvar notmuch-indicator--counters nil
+  "Store the return value of `notmuch-indicator--indicator'.")
 
 (defun notmuch-indicator--indicator ()
-  "Prepare `notmuch-indicator-string'."
-  (setq global-mode-string (delq 'notmuch-indicator-string global-mode-string))
-  (if-let ((count (notmuch-indicator--return-single-string)))
-      (setq notmuch-indicator-string count
-            ;; FIXME 2022-09-22: This may be hacky, but I cannot remember or
-            ;; find a function that appends an element as the second in a
-            ;; list.  I don't want it to be at the very start or end...
-            global-mode-string
-            (reverse
-             (append
-              (butlast (reverse global-mode-string))
-              '(notmuch-indicator-string)
-              '(""))))
-    (setq notmuch-indicator-string ""))
-  (force-mode-line-update t))
+  "Return contents of mode line indicator."
+  (setq notmuch-indicator--counters
+        (or (notmuch-indicator--get-counters) "")))
+
+(defvar-local notmuch-indicator-mode-line-construct
+    '(notmuch-indicator-mode (" " (:eval notmuch-indicator--counters)))
+  "Show the notmuch-indicator on the mode line.
+Do it when `notmuch-indicator-mode' is enabled.  Also see
+`notmuch-indicator-add-to-mode-line-misc-info'.")
+
+(put 'notmuch-indicator-mode-line-construct 'risky-local-variable t)
+
+(defun notmuch-indicator-tab-bar-format ()
+  "Notmuch indicator construct suitable for `tab-bar-format'."
+  `((global menu-item ,(format-mode-line notmuch-indicator-mode-line-construct) ignore)))
 
 (defun notmuch-indicator--running-p ()
   "Return non-nil if `notmuch-indicator--indicator' is running."
@@ -290,20 +329,22 @@ It is appended to the `global-mode-string'.")
   "Run the timer with a delay, starting it if necessary.
 The delay is specified by `notmuch-indicator-refresh-count'."
   (unless (notmuch-indicator--running-p)
-    (notmuch-indicator--indicator)
-    (run-at-time t notmuch-indicator-refresh-count #'notmuch-indicator--indicator)))
+    (run-at-time nil notmuch-indicator-refresh-count #'notmuch-indicator--indicator)))
 
 (defun notmuch-indicator-refresh ()
   "Refresh the active indicator."
   (when (notmuch-indicator--running-p)
     (cancel-function-timers #'notmuch-indicator--indicator)
-    (notmuch-indicator--run)))
+    (run-at-time nil notmuch-indicator-refresh-count #'notmuch-indicator--indicator)))
 
 (define-obsolete-function-alias
   'notmuch-indicator--refresh
   'notmuch-indicator-refresh
   "0.3.0")
 
+(defvar notmuch-indicator--used-mode-line-construct nil
+  "Mode line construct last added by `notmuch-indicator-mode'.")
+
 ;;;###autoload
 (define-minor-mode notmuch-indicator-mode
   "Display mode line indicator with `notmuch-count(1)' output.
@@ -317,11 +358,14 @@ option `notmuch-indicator-refresh-count'.."
   :global t
   (if notmuch-indicator-mode
       (progn
+        (when notmuch-indicator-add-to-mode-line-misc-info
+          (setq notmuch-indicator--used-mode-line-construct notmuch-indicator-mode-line-construct)
+          (add-to-list 'mode-line-misc-info notmuch-indicator-mode-line-construct))
         (notmuch-indicator--run)
         (dolist (fn notmuch-indicator-force-refresh-commands)
           (advice-add fn :after #'notmuch-indicator-refresh)))
+    (setq mode-line-misc-info (delete notmuch-indicator--used-mode-line-construct mode-line-misc-info))
     (cancel-function-timers #'notmuch-indicator--indicator)
-    (setq global-mode-string (delq 'notmuch-indicator-string global-mode-string))
     (dolist (fn notmuch-indicator-force-refresh-commands)
       (advice-remove fn #'notmuch-indicator-refresh))
     (force-mode-line-update t)))
